@@ -242,7 +242,7 @@ def _run(cmdline_args=None, main_task_cls=None,
     if main_task_cls:
         cmdline_args.insert(0, main_task_cls.task_family)
     if local_scheduler:
-        cmdline_args.insert(0, '--local-scheduler')
+        cmdline_args.append('--local-scheduler')
 
     with CmdlineParser.global_instance(cmdline_args) as cp:
         return _schedule_and_run([cp.get_task_obj()], worker_scheduler_factory)
